@@ -1,20 +1,20 @@
 /* Any copyright is dedicated to the Public Domain.
  * http://creativecommons.org/publicdomain/zero/1.0/ */
 
-let { LoginBreaches } = ChromeUtils.import(
-  "resource:///modules/LoginBreaches.jsm"
+let { LoginBreaches } = ChromeUtils.importESModule(
+  "resource:///modules/LoginBreaches.sys.mjs"
 );
-let { RemoteSettings } = ChromeUtils.import(
-  "resource://services-settings/remote-settings.js"
+let { RemoteSettings } = ChromeUtils.importESModule(
+  "resource://services-settings/remote-settings.sys.mjs"
 );
-let { _AboutLogins } = ChromeUtils.import(
-  "resource:///actors/AboutLoginsParent.jsm"
+let { _AboutLogins } = ChromeUtils.importESModule(
+  "resource:///actors/AboutLoginsParent.sys.mjs"
 );
-let { OSKeyStoreTestUtils } = ChromeUtils.import(
-  "resource://testing-common/OSKeyStoreTestUtils.jsm"
+let { OSKeyStoreTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/OSKeyStoreTestUtils.sys.mjs"
 );
-let { LoginTestUtils } = ChromeUtils.import(
-  "resource://testing-common/LoginTestUtils.jsm"
+var { LoginTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/LoginTestUtils.sys.mjs"
 );
 
 let nsLoginInfo = new Components.Constructor(
@@ -54,17 +54,12 @@ let TEST_LOGIN3 = new nsLoginInfo(
 TEST_LOGIN3.QueryInterface(Ci.nsILoginMetaInfo).timePasswordChanged = 123456;
 
 async function addLogin(login) {
-  let storageChangedPromised = TestUtils.topicObserved(
-    "passwordmgr-storage-changed",
-    (_, data) => data == "addLogin"
-  );
-  login = Services.logins.addLogin(login);
-  await storageChangedPromised;
+  const result = await Services.logins.addLoginAsync(login);
   registerCleanupFunction(() => {
     let matchData = Cc["@mozilla.org/hash-property-bag;1"].createInstance(
       Ci.nsIWritablePropertyBag2
     );
-    matchData.setPropertyAsAUTF8String("guid", login.guid);
+    matchData.setPropertyAsAUTF8String("guid", result.guid);
 
     let logins = Services.logins.searchLogins(matchData);
     if (!logins.length) {
@@ -76,19 +71,19 @@ async function addLogin(login) {
     // matches the login that it will be removing.
     Services.logins.removeLogin(logins[0]);
   });
-  return login;
+  return result;
 }
 
 let EXPECTED_BREACH = null;
 let EXPECTED_ERROR_MESSAGE = null;
-add_task(async function setup_head() {
-  const db = await RemoteSettings(LoginBreaches.REMOTE_SETTINGS_COLLECTION).db;
+add_setup(async function setup_head() {
+  const db = RemoteSettings(LoginBreaches.REMOTE_SETTINGS_COLLECTION).db;
   if (EXPECTED_BREACH) {
     await db.create(EXPECTED_BREACH, {
       useRecordId: true,
     });
   }
-  await db.importChanges({}, 42);
+  await db.importChanges({}, Date.now());
   if (EXPECTED_BREACH) {
     await RemoteSettings(LoginBreaches.REMOTE_SETTINGS_COLLECTION).emit(
       "sync",
@@ -103,7 +98,7 @@ add_task(async function setup_head() {
     }
 
     if (msg.errorMessage.includes('Unknown event: ["jsonfile", "load"')) {
-      // Ignore telemetry errors from JSONFile.jsm.
+      // Ignore telemetry errors from JSONFile.sys.mjs.
       return;
     }
 
@@ -151,7 +146,14 @@ add_task(async function setup_head() {
       // Ignore errors handled by the error message dialog.
       return;
     }
-    ok(false, msg.message || msg.errorMessage);
+    if (
+      msg.errorMessage ==
+      "NotFoundError: No such JSWindowActor 'MarionetteEvents'"
+    ) {
+      // Ignore MarionetteEvents error (Bug 1730837, Bug 1710079).
+      return;
+    }
+    Assert.ok(false, msg.message || msg.errorMessage);
   });
 
   registerCleanupFunction(async () => {
@@ -163,29 +165,33 @@ add_task(async function setup_head() {
 });
 
 /**
- * Waits for the master password prompt and performs an action.
+ * Waits for the primary password prompt and performs an action.
  * @param {string} action Set to "authenticate" to log in or "cancel" to
  *        close the dialog without logging in.
  */
-function waitForMPDialog(action) {
+function waitForMPDialog(action, aWindow = window) {
   const BRAND_BUNDLE = Services.strings.createBundle(
     "chrome://branding/locale/brand.properties"
   );
   const BRAND_FULL_NAME = BRAND_BUNDLE.GetStringFromName("brandFullName");
   let dialogShown = TestUtils.topicObserved("common-dialog-loaded");
-  return dialogShown.then(function([subject]) {
+  return dialogShown.then(function ([subject]) {
     let dialog = subject.Dialog;
     let expected = "Password Required - " + BRAND_FULL_NAME;
-    is(dialog.args.title, expected, "Dialog is the Master Password dialog");
+    Assert.equal(
+      dialog.args.title,
+      expected,
+      "Dialog is the Primary Password dialog"
+    );
     if (action == "authenticate") {
       SpecialPowers.wrap(dialog.ui.password1Textbox).setUserInput(
-        LoginTestUtils.masterPassword.masterPassword
+        LoginTestUtils.primaryPassword.primaryPassword
       );
       dialog.ui.button0.click();
     } else if (action == "cancel") {
       dialog.ui.button1.click();
     }
-    return BrowserTestUtils.waitForEvent(window, "DOMModalDialogClosed");
+    return BrowserTestUtils.waitForEvent(aWindow, "DOMModalDialogClosed");
   });
 }
 
@@ -198,10 +204,10 @@ function waitForMPDialog(action) {
  *        close the dialog without logging in.
  * @returns {Promise} Resolves after the MP dialog has been presented and actioned upon
  */
-function forceAuthTimeoutAndWaitForMPDialog(action) {
-  const AUTH_TIMEOUT_MS = 5 * 60 * 1000; // 5 minutes (duplicated from AboutLoginsParent.jsm)
+function forceAuthTimeoutAndWaitForMPDialog(action, aWindow = window) {
+  const AUTH_TIMEOUT_MS = 5 * 60 * 1000; // 5 minutes (duplicated from AboutLoginsParent.sys.mjs)
   _AboutLogins._authExpirationTime -= AUTH_TIMEOUT_MS + 1;
-  return waitForMPDialog(action);
+  return waitForMPDialog(action, aWindow);
 }
 
 /**
@@ -213,7 +219,7 @@ function forceAuthTimeoutAndWaitForMPDialog(action) {
  * @returns {Promise} Resolves after the OS auth dialog has been presented
  */
 function forceAuthTimeoutAndWaitForOSKeyStoreLogin({ loginResult }) {
-  const AUTH_TIMEOUT_MS = 5 * 60 * 1000; // 5 minutes (duplicated from AboutLoginsParent.jsm)
+  const AUTH_TIMEOUT_MS = 5 * 60 * 1000; // 5 minutes (duplicated from AboutLoginsParent.sys.mjs)
   _AboutLogins._authExpirationTime -= AUTH_TIMEOUT_MS + 1;
   return OSKeyStoreTestUtils.waitForOSKeyStoreLogin(loginResult);
 }
