@@ -6,12 +6,12 @@
 
 "use strict";
 
-var { XPCOMUtils } = ChromeUtils.import(
-  "resource://gre/modules/XPCOMUtils.jsm"
+var { XPCOMUtils } = ChromeUtils.importESModule(
+  "resource://gre/modules/XPCOMUtils.sys.mjs"
 );
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  PlacesUtils: "resource://gre/modules/PlacesUtils.jsm",
+ChromeUtils.defineESModuleGetters(this, {
+  PlacesUtils: "resource://gre/modules/PlacesUtils.sys.mjs",
 });
 
 var { ExtensionError } = ExtensionUtils;
@@ -120,15 +120,9 @@ const throwIfRootId = id => {
 let observer = new (class extends EventEmitter {
   constructor() {
     super();
-
-    this.skipTags = true;
-
     this.handlePlacesEvents = this.handlePlacesEvents.bind(this);
   }
 
-  onBeginUpdateBatch() {}
-  onEndUpdateBatch() {}
-
   handlePlacesEvents(events) {
     for (let event of events) {
       switch (event.type) {
@@ -162,72 +156,61 @@ let observer = new (class extends EventEmitter {
             index: event.index,
             type: BOOKMARKS_TYPES_TO_API_TYPES_MAP.get(event.itemType),
             url: getUrl(event.itemType, event.url),
+            title: event.title,
           };
 
           this.emit("removed", {
             guid: event.guid,
             info: { parentId: event.parentGuid, index: event.index, node },
           });
+          break;
+        case "bookmark-moved":
+          this.emit("moved", {
+            guid: event.guid,
+            info: {
+              parentId: event.parentGuid,
+              index: event.index,
+              oldParentId: event.oldParentGuid,
+              oldIndex: event.oldIndex,
+            },
+          });
+          break;
+        case "bookmark-title-changed":
+          if (event.isTagging) {
+            continue;
+          }
+
+          this.emit("changed", {
+            guid: event.guid,
+            info: { title: event.title },
+          });
+          break;
+        case "bookmark-url-changed":
+          if (event.isTagging) {
+            continue;
+          }
+
+          this.emit("changed", {
+            guid: event.guid,
+            info: { url: event.url },
+          });
+          break;
       }
     }
   }
-
-  onItemVisited() {}
-
-  onItemMoved(
-    id,
-    oldParentId,
-    oldIndex,
-    newParentId,
-    newIndex,
-    itemType,
-    guid,
-    oldParentGuid,
-    newParentGuid,
-    source
-  ) {
-    let info = {
-      parentId: newParentGuid,
-      index: newIndex,
-      oldParentId: oldParentGuid,
-      oldIndex,
-    };
-    this.emit("moved", { guid, info });
-  }
-
-  onItemChanged(
-    id,
-    prop,
-    isAnno,
-    val,
-    lastMod,
-    itemType,
-    parentId,
-    guid,
-    parentGuid,
-    oldVal,
-    source
-  ) {
-    let info = {};
-    if (prop == "title") {
-      info.title = val;
-    } else if (prop == "uri") {
-      info.url = val;
-    } else {
-      // Not defined yet.
-      return;
-    }
-
-    this.emit("changed", { guid, info });
-  }
 })();
 
 const decrementListeners = () => {
   listenerCount -= 1;
   if (!listenerCount) {
-    PlacesUtils.bookmarks.removeObserver(observer);
     PlacesUtils.observers.removeListener(
-      ["bookmark-added", "bookmark-removed"],
+      [
+        "bookmark-added",
+        "bookmark-removed",
+        "bookmark-moved",
+        "bookmark-title-changed",
+        "bookmark-url-changed",
+      ],
       observer.handlePlacesEvents
     );
   }
@@ -236,15 +219,94 @@ const decrementListeners = () => {
 const incrementListeners = () => {
   listenerCount++;
   if (listenerCount == 1) {
-    PlacesUtils.bookmarks.addObserver(observer);
     PlacesUtils.observers.addListener(
-      ["bookmark-added", "bookmark-removed"],
+      [
+        "bookmark-added",
+        "bookmark-removed",
+        "bookmark-moved",
+        "bookmark-title-changed",
+        "bookmark-url-changed",
+      ],
       observer.handlePlacesEvents
     );
   }
 };
 
-this.bookmarks = class extends ExtensionAPI {
+this.bookmarks = class extends ExtensionAPIPersistent {
+  PERSISTENT_EVENTS = {
+    onCreated({ fire }) {
+      let listener = (event, bookmark) => {
+        fire.sync(bookmark.id, bookmark);
+      };
+
+      observer.on("created", listener);
+      incrementListeners();
+      return {
+        unregister() {
+          observer.off("created", listener);
+          decrementListeners();
+        },
+        convert(_fire) {
+          fire = _fire;
+        },
+      };
+    },
+
+    onRemoved({ fire }) {
+      let listener = (event, data) => {
+        fire.sync(data.guid, data.info);
+      };
+
+      observer.on("removed", listener);
+      incrementListeners();
+      return {
+        unregister() {
+          observer.off("removed", listener);
+          decrementListeners();
+        },
+        convert(_fire) {
+          fire = _fire;
+        },
+      };
+    },
+
+    onChanged({ fire }) {
+      let listener = (event, data) => {
+        fire.sync(data.guid, data.info);
+      };
+
+      observer.on("changed", listener);
+      incrementListeners();
+      return {
+        unregister() {
+          observer.off("changed", listener);
+          decrementListeners();
+        },
+        convert(_fire) {
+          fire = _fire;
+        },
+      };
+    },
+
+    onMoved({ fire }) {
+      let listener = (event, data) => {
+        fire.sync(data.guid, data.info);
+      };
+
+      observer.on("moved", listener);
+      incrementListeners();
+      return {
+        unregister() {
+          observer.off("moved", listener);
+          decrementListeners();
+        },
+        convert(_fire) {
+          fire = _fire;
+        },
+      };
+    },
+  };
+
   getAPI(context) {
     return {
       bookmarks: {
@@ -266,32 +328,32 @@ this.bookmarks = class extends ExtensionAPI {
           }
         },
 
-        getChildren: function(id) {
+        getChildren: function (id) {
           // TODO: We should optimize this.
           return getTree(id, true);
         },
 
-        getTree: function() {
+        getTree: function () {
           return getTree(PlacesUtils.bookmarks.rootGuid, false);
         },
 
-        getSubTree: function(id) {
+        getSubTree: function (id) {
           return getTree(id, false);
         },
 
-        search: function(query) {
+        search: function (query) {
           return PlacesUtils.bookmarks
             .search(query)
             .then(result => result.map(convertBookmarks));
         },
 
-        getRecent: function(numberOfItems) {
+        getRecent: function (numberOfItems) {
           return PlacesUtils.bookmarks
             .getRecent(numberOfItems)
             .then(result => result.map(convertBookmarks));
         },
 
-        create: function(bookmark) {
+        create: function (bookmark) {
           let info = {
             title: bookmark.title || "",
           };
@@ -333,7 +395,7 @@ this.bookmarks = class extends ExtensionAPI {
           }
         },
 
-        move: function(id, destination) {
+        move: function (id, destination) {
           throwIfRootId(id);
           let info = {
             guid: id,
@@ -360,7 +422,7 @@ this.bookmarks = class extends ExtensionAPI {
           }
         },
 
-        update: function(id, changes) {
+        update: function (id, changes) {
           throwIfRootId(id);
           let info = {
             guid: id,
@@ -385,7 +447,7 @@ this.bookmarks = class extends ExtensionAPI {
           }
         },
 
-        remove: function(id) {
+        remove: function (id) {
           throwIfRootId(id);
           let info = {
             guid: id,
@@ -403,7 +465,7 @@ this.bookmarks = class extends ExtensionAPI {
           }
         },
 
-        removeTree: function(id) {
+        removeTree: function (id) {
           throwIfRootId(id);
           let info = {
             guid: id,
@@ -422,70 +484,30 @@ this.bookmarks = class extends ExtensionAPI {
 
         onCreated: new EventManager({
           context,
-          name: "bookmarks.onCreated",
-          register: fire => {
-            let listener = (event, bookmark) => {
-              fire.sync(bookmark.id, bookmark);
-            };
-
-            observer.on("created", listener);
-            incrementListeners();
-            return () => {
-              observer.off("created", listener);
-              decrementListeners();
-            };
-          },
+          module: "bookmarks",
+          event: "onCreated",
+          extensionApi: this,
         }).api(),
 
         onRemoved: new EventManager({
           context,
-          name: "bookmarks.onRemoved",
-          register: fire => {
-            let listener = (event, data) => {
-              fire.sync(data.guid, data.info);
-            };
-
-            observer.on("removed", listener);
-            incrementListeners();
-            return () => {
-              observer.off("removed", listener);
-              decrementListeners();
-            };
-          },
+          module: "bookmarks",
+          event: "onRemoved",
+          extensionApi: this,
         }).api(),
 
         onChanged: new EventManager({
           context,
-          name: "bookmarks.onChanged",
-          register: fire => {
-            let listener = (event, data) => {
-              fire.sync(data.guid, data.info);
-            };
-
-            observer.on("changed", listener);
-            incrementListeners();
-            return () => {
-              observer.off("changed", listener);
-              decrementListeners();
-            };
-          },
+          module: "bookmarks",
+          event: "onChanged",
+          extensionApi: this,
         }).api(),
 
         onMoved: new EventManager({
           context,
-          name: "bookmarks.onMoved",
-          register: fire => {
-            let listener = (event, data) => {
-              fire.sync(data.guid, data.info);
-            };
-
-            observer.on("moved", listener);
-            incrementListeners();
-            return () => {
-              observer.off("moved", listener);
-              decrementListeners();
-            };
-          },
+          module: "bookmarks",
+          event: "onMoved",
+          extensionApi: this,
         }).api(),
       },
     };
