@@ -1,5 +1,7 @@
 import { mount } from "enzyme";
 import React from "react";
+import { FluentBundle, FluentResource } from "@fluent/bundle";
+import { LocalizationProvider, ReactLocalization } from "@fluent/react";
 import schema from "content-src/asrouter/templates/SimpleSnippet/SimpleSnippet.schema.json";
 import { SimpleSnippet } from "content-src/asrouter/templates/SimpleSnippet/SimpleSnippet.jsx";
 
@@ -10,6 +12,20 @@ describe("SimpleSnippet", () => {
   let onBlockStub;
   let sendUserActionTelemetryStub;
 
+  function mockL10nWrapper(content) {
+    const bundle = new FluentBundle("en-US");
+    for (const [id, value] of Object.entries(content)) {
+      if (typeof value === "string") {
+        bundle.addResource(new FluentResource(`${id} = ${value}`));
+      }
+    }
+    const l10n = new ReactLocalization([bundle]);
+    return {
+      wrappingComponent: LocalizationProvider,
+      wrappingComponentProps: { l10n },
+    };
+  }
+
   /**
    * mountAndCheckProps - Mounts a SimpleSnippet with DEFAULT_CONTENT extended with any props
    *                      passed in the content param and validates props against the schema.
@@ -25,7 +41,7 @@ describe("SimpleSnippet", () => {
       onAction: sandbox.stub(),
     };
     assert.jsonSchema(props.content, schema);
-    return mount(<SimpleSnippet {...props} />);
+    return mount(<SimpleSnippet {...props} />, mockL10nWrapper(props.content));
   }
 
   beforeEach(() => {
@@ -56,13 +72,7 @@ describe("SimpleSnippet", () => {
   });
   it("should render .title", () => {
     const wrapper = mountAndCheckProps({ title: "Foo" });
-    assert.equal(
-      wrapper
-        .find(".title")
-        .text()
-        .trim(),
-      "Foo"
-    );
+    assert.equal(wrapper.find(".title").text().trim(), "Foo");
   });
   it("should render a light theme variant .icon", () => {
     const wrapper = mountAndCheckProps({
@@ -134,10 +144,7 @@ describe("SimpleSnippet", () => {
       'url("data:image/gif;base64,R0lGODl")'
     );
     assert.equal(
-      wrapper
-        .find(".section-title-text")
-        .text()
-        .trim(),
+      wrapper.find(".section-title-text").text().trim(),
       "Messages from Mozilla"
     );
     // ensure there is no <a> when a section_title_url is not specified
@@ -156,10 +163,7 @@ describe("SimpleSnippet", () => {
       'url("data:image/gif;base64,R0lGODl")'
     );
     assert.equal(
-      wrapper
-        .find(".section-title-text")
-        .text()
-        .trim(),
+      wrapper.find(".section-title-text").text().trim(),
       "Messages from Mozilla"
     );
     // ensure there is no <a> when a section_title_url is not specified
