@@ -5,45 +5,20 @@
 /* import-globals-from extensionControlled.js */
 /* import-globals-from preferences.js */
 
-ChromeUtils.defineModuleGetter(
-  this,
-  "PlacesUtils",
-  "resource://gre/modules/PlacesUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "ExtensionSettingsStore",
-  "resource://gre/modules/ExtensionSettingsStore.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "UrlbarPrefs",
-  "resource:///modules/UrlbarPrefs.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "UrlbarUtils",
-  "resource:///modules/UrlbarUtils.jsm"
-);
-ChromeUtils.defineModuleGetter(
-  this,
-  "NimbusFeatures",
-  "resource://nimbus/ExperimentAPI.jsm"
-);
+const lazy = {};
 
-XPCOMUtils.defineLazyModuleGetters(this, {
-  UrlbarProviderQuickSuggest:
-    "resource:///modules/UrlbarProviderQuickSuggest.jsm",
+ChromeUtils.defineESModuleGetters(lazy, {
+  SearchUIUtils: "resource:///modules/SearchUIUtils.sys.mjs",
 });
 
 Preferences.addAll([
   { id: "browser.search.suggest.enabled", type: "bool" },
   { id: "browser.urlbar.suggest.searches", type: "bool" },
   { id: "browser.search.suggest.enabled.private", type: "bool" },
-  { id: "browser.urlbar.suggest.quicksuggest", type: "bool" },
   { id: "browser.search.hiddenOneOffs", type: "unichar" },
   { id: "browser.search.widget.inNavBar", type: "bool" },
   { id: "browser.urlbar.showSearchSuggestionsFirst", type: "bool" },
+  { id: "browser.urlbar.showSearchTerms.enabled", type: "bool" },
   { id: "browser.search.separatePrivateDefault", type: "bool" },
   { id: "browser.search.separatePrivateDefault.ui.enabled", type: "bool" },
 ]);
@@ -67,10 +42,7 @@ var gSearchPane = {
       document.getElementById("addEnginesBox").hidden = true;
     } else {
       let addEnginesLink = document.getElementById("addEngines");
-      let searchEnginesURL = Services.wm.getMostRecentWindow(
-        "navigator:browser"
-      ).BrowserSearch.searchEnginesURL;
-      addEnginesLink.setAttribute("href", searchEnginesURL);
+      addEnginesLink.setAttribute("href", lazy.SearchUIUtils.searchEnginesURL);
     }
 
     window.addEventListener("click", this);
@@ -81,8 +53,10 @@ var gSearchPane = {
     window.addEventListener("dblclick", this);
 
     Services.obs.addObserver(this, "browser-search-engine-modified");
+    Services.obs.addObserver(this, "intl:app-locales-changed");
     window.addEventListener("unload", () => {
       Services.obs.removeObserver(this, "browser-search-engine-modified");
+      Services.obs.removeObserver(this, "intl:app-locales-changed");
     });
 
     let suggestsPref = Preferences.get("browser.search.suggest.enabled");
@@ -90,9 +64,8 @@ var gSearchPane = {
     let privateSuggestsPref = Preferences.get(
       "browser.search.suggest.enabled.private"
     );
-    let updateSuggestionCheckboxes = this._updateSuggestionCheckboxes.bind(
-      this
-    );
+    let updateSuggestionCheckboxes =
+      this._updateSuggestionCheckboxes.bind(this);
     suggestsPref.on("change", updateSuggestionCheckboxes);
     urlbarSuggestsPref.on("change", updateSuggestionCheckboxes);
     let urlbarSuggests = document.getElementById("urlBarSuggestion");
@@ -111,23 +84,20 @@ var gSearchPane = {
       "command",
       this._onBrowserSeparateDefaultEngineChange.bind(this)
     );
-    setEventListener("openLocationBarPrivacyPreferences", "click", function(
-      event
-    ) {
-      if (event.button == 0) {
-        gotoPref("privacy-locationBar");
+    setEventListener(
+      "openLocationBarPrivacyPreferences",
+      "click",
+      function (event) {
+        if (event.button == 0) {
+          gotoPref("privacy-locationBar");
+        }
       }
-    });
+    );
 
     this._initDefaultEngines();
+    this._initShowSearchTermsCheckbox();
     this._updateSuggestionCheckboxes();
     this._showAddEngineButton();
-    this._updateQuickSuggest = this._updateQuickSuggest.bind(this);
-    NimbusFeatures.urlbar.onUpdate(this._updateQuickSuggest);
-    window.addEventListener("unload", () => {
-      NimbusFeatures.urlbar.off(this._updateQuickSuggest);
-    });
-    this._updateQuickSuggest(true);
   },
 
   /**
@@ -157,11 +127,36 @@ var gSearchPane = {
     this._separatePrivateDefaultPref.on("change", listener);
   },
 
+  _initShowSearchTermsCheckbox() {
+    let checkbox = document.getElementById("searchShowSearchTermCheckbox");
+
+    // Add Nimbus event to show/hide checkbox.
+    let onNimbus = () => {
+      checkbox.hidden = !UrlbarPrefs.get("showSearchTermsFeatureGate");
+    };
+    NimbusFeatures.urlbar.onUpdate(onNimbus);
+
+    // Add observer of Search Bar preference as showSearchTerms
+    // can't be enabled/disabled while Search Bar is enabled.
+    let searchBarPref = Preferences.get("browser.search.widget.inNavBar");
+    let updateCheckboxEnabled = () => {
+      checkbox.disabled = searchBarPref.value;
+    };
+    searchBarPref.on("change", updateCheckboxEnabled);
+
+    // Fire once to initialize.
+    onNimbus();
+    updateCheckboxEnabled();
+
+    window.addEventListener("unload", () => {
+      NimbusFeatures.urlbar.offUpdate(onNimbus);
+    });
+  },
+
   _updatePrivateEngineDisplayBoxes() {
     const separateEnabled = this._separatePrivateDefaultEnabledPref.value;
-    document.getElementById(
-      "browserSeparateDefaultEngine"
-    ).hidden = !separateEnabled;
+    document.getElementById("browserSeparateDefaultEngine").hidden =
+      !separateEnabled;
 
     const separateDefault = this._separatePrivateDefaultPref.value;
 
@@ -185,7 +180,6 @@ var gSearchPane = {
     let privateWindowCheckbox = document.getElementById(
       "showSearchSuggestionsPrivateWindows"
     );
-    let quickSuggestCheckbox = document.getElementById("showQuickSuggest");
 
     urlbarSuggests.disabled = !suggestsPref.value || permanentPB;
     privateWindowCheckbox.disabled = !suggestsPref.value;
@@ -209,15 +203,9 @@ var gSearchPane = {
       positionCheckbox.checked = Preferences.get(
         positionCheckbox.getAttribute("preference")
       ).value;
-      quickSuggestCheckbox.disabled = false;
-      quickSuggestCheckbox.checked = Preferences.get(
-        quickSuggestCheckbox.getAttribute("preference")
-      ).value;
     } else {
       positionCheckbox.disabled = true;
       positionCheckbox.checked = false;
-      quickSuggestCheckbox.disabled = true;
-      quickSuggestCheckbox.checked = false;
     }
 
     let permanentPBLabel = document.getElementById(
@@ -226,44 +214,6 @@ var gSearchPane = {
     permanentPBLabel.hidden = urlbarSuggests.hidden || !permanentPB;
   },
 
-  /**
-   * Shows or hides the Quick Suggest checkbox depending on whether the en-US
-   * Quick Suggest experiment is enabled.
-   *
-   * @param {boolean} [onStartup]
-   *   True when this is called from `.init`
-   */
-  _updateQuickSuggest(onStartup = false) {
-    let container = document.getElementById("showQuickSuggestContainer");
-    let desc = document.getElementById("searchSuggestionsDesc");
-
-    if (!UrlbarPrefs.get("quickSuggestEnabled")) {
-      // The experiment is not enabled.  This is the default, so to avoid
-      // accidentally messing anything up, only modify the doc if we're being
-      // called due to a change in the experiment enabled status.
-      if (!onStartup) {
-        container.setAttribute("hidden", "true");
-        if (desc.dataset.l10nIdOriginal) {
-          desc.dataset.l10nId = desc.dataset.l10nIdOriginal;
-          delete desc.dataset.l10nIdOriginal;
-        }
-        document.l10n.translateElements([desc]);
-      }
-      return;
-    }
-
-    // The experiment is enabled.
-    document
-      .getElementById("showQuickSuggestLearnMore")
-      .setAttribute("href", UrlbarProviderQuickSuggest.helpUrl);
-    container.removeAttribute("hidden");
-    if (desc.dataset.l10nId) {
-      desc.dataset.l10nIdOriginal = desc.dataset.l10nId;
-      delete desc.dataset.l10nId;
-    }
-    desc.textContent = "Choose how search suggestions appear.";
-  },
-
   _showAddEngineButton() {
     let aliasRefresh = Services.prefs.getBoolPref(
       "browser.urlbar.update2.engineAliasRefresh",
@@ -282,14 +232,18 @@ var gSearchPane = {
   async buildDefaultEngineDropDowns() {
     await this._buildEngineDropDown(
       document.getElementById("defaultEngine"),
-      (await Services.search.getDefault()).name,
+      (
+        await Services.search.getDefault()
+      ).name,
       false
     );
 
     if (this._separatePrivateDefaultEnabledPref.value) {
       await this._buildEngineDropDown(
         document.getElementById("defaultPrivateEngine"),
-        (await Services.search.getDefaultPrivate()).name,
+        (
+          await Services.search.getDefaultPrivate()
+        ).name,
         true
       );
     }
@@ -360,7 +314,7 @@ var gSearchPane = {
           // away from.
           if (engineList.inputField.hidden && engineList.view) {
             let selection = engineList.view.selection;
-            if (selection.count > 0) {
+            if (selection?.count > 0) {
               selection.toggleSelect(selection.currentIndex);
             }
             engineList.blur();
@@ -418,65 +372,85 @@ var gSearchPane = {
   },
 
   /**
-   * nsIObserver implementation.  We observe the following:
-   *
-   * * browser-search-engine-modified: Update the default engine UI and engine
-   *   tree view as appropriate when engine changes occur.
+   * Handle when the app locale is changed.
    */
-  observe(subject, topic, data) {
-    if (topic == "browser-search-engine-modified") {
-      let engine = subject;
-      engine.QueryInterface(Ci.nsISearchEngine);
-      switch (data) {
-        case "engine-added":
-          gEngineView._engineStore.addEngine(engine);
-          gEngineView.rowCountChanged(gEngineView.lastEngineIndex, 1);
+  async appLocalesChanged() {
+    await document.l10n.ready;
+    await gEngineView.loadL10nNames();
+  },
+
+  /**
+   * Update the default engine UI and engine tree view as appropriate when engine changes
+   * or locale changes occur.
+   *
+   * @param {Object} engine
+   * @param {string} data
+   */
+  browserSearchEngineModified(engine, data) {
+    engine.QueryInterface(Ci.nsISearchEngine);
+    switch (data) {
+      case "engine-added":
+        gEngineView._engineStore.addEngine(engine);
+        gEngineView.rowCountChanged(gEngineView.lastEngineIndex, 1);
+        gSearchPane.buildDefaultEngineDropDowns();
+        break;
+      case "engine-changed":
+        gSearchPane.buildDefaultEngineDropDowns();
+        gEngineView._engineStore.updateEngine(engine);
+        gEngineView.invalidate();
+        break;
+      case "engine-removed":
+        gSearchPane.remove(engine);
+        break;
+      case "engine-default": {
+        // If the user is going through the drop down using up/down keys, the
+        // dropdown may still be open (eg. on Windows) when engine-default is
+        // fired, so rebuilding the list unconditionally would get in the way.
+        let selectedEngine =
+          document.getElementById("defaultEngine").selectedItem.engine;
+        if (selectedEngine.name != engine.name) {
           gSearchPane.buildDefaultEngineDropDowns();
-          break;
-        case "engine-changed":
-          gSearchPane.buildDefaultEngineDropDowns();
-          gEngineView._engineStore.updateEngine(engine);
-          gEngineView.invalidate();
-          break;
-        case "engine-removed":
-          gSearchPane.remove(engine);
-          break;
-        case "engine-default": {
+        }
+        break;
+      }
+      case "engine-default-private": {
+        if (
+          this._separatePrivateDefaultEnabledPref.value &&
+          this._separatePrivateDefaultPref.value
+        ) {
           // If the user is going through the drop down using up/down keys, the
           // dropdown may still be open (eg. on Windows) when engine-default is
           // fired, so rebuilding the list unconditionally would get in the way.
-          let selectedEngine = document.getElementById("defaultEngine")
+          const selectedEngine = document.getElementById("defaultPrivateEngine")
             .selectedItem.engine;
           if (selectedEngine.name != engine.name) {
             gSearchPane.buildDefaultEngineDropDowns();
           }
-          break;
-        }
-        case "engine-default-private": {
-          if (
-            this._separatePrivateDefaultEnabledPref.value &&
-            this._separatePrivateDefaultPref.value
-          ) {
-            // If the user is going through the drop down using up/down keys, the
-            // dropdown may still be open (eg. on Windows) when engine-default is
-            // fired, so rebuilding the list unconditionally would get in the way.
-            const selectedEngine = document.getElementById(
-              "defaultPrivateEngine"
-            ).selectedItem.engine;
-            if (selectedEngine.name != engine.name) {
-              gSearchPane.buildDefaultEngineDropDowns();
-            }
-          }
-          break;
         }
+        break;
+      }
+    }
+  },
+
+  /**
+   * nsIObserver implementation.
+   */
+  observe(subject, topic, data) {
+    switch (topic) {
+      case "intl:app-locales-changed": {
+        this.appLocalesChanged();
+        break;
+      }
+      case "browser-search-engine-modified": {
+        this.browserSearchEngineModified(subject, data);
+        break;
       }
     }
   },
 
   onTreeSelect() {
-    document.getElementById(
-      "removeEngineButton"
-    ).disabled = !gEngineView.isEngineSelectedAndRemovable();
+    document.getElementById("removeEngineButton").disabled =
+      !gEngineView.isEngineSelectedAndRemovable();
   },
 
   onTreeKeyPress(aEvent) {
@@ -544,10 +518,17 @@ var gSearchPane = {
 
   remove(aEngine) {
     let index = gEngineView._engineStore.removeEngine(aEngine);
+    if (!gEngineView.tree) {
+      // Only update the selection if it's visible in the UI.
+      return;
+    }
+
     gEngineView.rowCountChanged(index, -1);
     gEngineView.invalidate();
+
     gEngineView.selection.select(Math.min(index, gEngineView.rowCount - 1));
     gEngineView.ensureRowIsVisible(gEngineView.currentIndex);
+
     document.getElementById("engineList").focus();
   },
 
@@ -606,14 +587,14 @@ var gSearchPane = {
         hiddenList.push(engine.name);
       }
     }
-    Preferences.get("browser.search.hiddenOneOffs").value = hiddenList.join(
-      ","
-    );
+    Preferences.get("browser.search.hiddenOneOffs").value =
+      hiddenList.join(",");
   },
 
   async setDefaultEngine() {
     await Services.search.setDefault(
-      document.getElementById("defaultEngine").selectedItem.engine
+      document.getElementById("defaultEngine").selectedItem.engine,
+      Ci.nsISearchService.CHANGE_REASON_USER
     );
     if (ExtensionSettingsStore.getSetting(SEARCH_TYPE, SEARCH_KEY) !== null) {
       ExtensionSettingsStore.select(
@@ -626,7 +607,8 @@ var gSearchPane = {
 
   async setDefaultPrivateEngine() {
     await Services.search.setDefaultPrivate(
-      document.getElementById("defaultPrivateEngine").selectedItem.engine
+      document.getElementById("defaultPrivateEngine").selectedItem.engine,
+      Ci.nsISearchService.CHANGE_REASON_USER
     );
   },
 };
@@ -691,7 +673,7 @@ EngineStore.prototype = {
 
   _cloneEngine(aEngine) {
     var clonedObj = {};
-    for (let i of ["name", "alias", "iconURI", "hidden"]) {
+    for (let i of ["id", "name", "alias", "iconURI", "hidden"]) {
       clonedObj[i] = aEngine[i];
     }
     clonedObj.originalEngine = aEngine;
@@ -701,7 +683,7 @@ EngineStore.prototype = {
 
   // Callback for Array's some(). A thisObj must be passed to some()
   _isSameEngine(aEngineClone) {
-    return aEngineClone.originalEngine == this.originalEngine;
+    return aEngineClone.originalEngine.id == this.originalEngine.id;
   },
 
   addEngine(aEngine) {
@@ -710,14 +692,11 @@ EngineStore.prototype = {
 
   updateEngine(newEngine) {
     let engineToUpdate = this._engines.findIndex(
-      e => e.originalEngine == newEngine
+      e => e.originalEngine.id == newEngine.id
     );
-    if (engineToUpdate == -1) {
-      console.error("Could not find engine to update");
-      return;
+    if (engineToUpdate != -1) {
+      this.engines[engineToUpdate] = this._cloneEngine(newEngine);
     }
-
-    this.engines[engineToUpdate] = this._cloneEngine(newEngine);
   },
 
   moveEngine(aEngine, aNewIndex) {
@@ -800,7 +779,7 @@ EngineStore.prototype = {
       }
     }
 
-    Services.search.resetToOriginalDefaultEngine();
+    Services.search.resetToAppDefaultEngine();
     gSearchPane.showRestoreDefaults(false);
     gSearchPane.buildDefaultEngineDropDowns();
     return added;
@@ -817,7 +796,7 @@ EngineStore.prototype = {
   },
 
   reloadIcons() {
-    this._engines.forEach(function(e) {
+    this._engines.forEach(function (e) {
       e.iconURI = e.originalEngine.iconURI;
     });
   },
@@ -828,31 +807,35 @@ function EngineView(aEngineStore) {
 
   UrlbarPrefs.addObserver(this);
 
-  // This maps local shortcut sources to their l10n names.  The names are needed
-  // by getCellText.  Getting the names is async but getCellText is not, so we
-  // cache them here to retrieve them syncronously in getCellText.
-  this._localShortcutL10nNames = new Map();
-  document.l10n
-    .formatValues(
-      UrlbarUtils.LOCAL_SEARCH_MODES.map(mode => {
-        let name = UrlbarUtils.getResultSourceName(mode.source);
-        return { id: `urlbar-search-mode-${name}` };
-      })
-    )
-    .then(names => {
-      for (let { source } of UrlbarUtils.LOCAL_SEARCH_MODES) {
-        this._localShortcutL10nNames.set(source, names.shift());
-      }
-      // Invalidate the tree now that we have the names in case getCellText was
-      // called before name retrieval finished.
-      this.invalidate();
-    });
+  this.loadL10nNames();
 }
 
 EngineView.prototype = {
   _engineStore: null,
   tree: null,
 
+  loadL10nNames() {
+    // This maps local shortcut sources to their l10n names.  The names are needed
+    // by getCellText.  Getting the names is async but getCellText is not, so we
+    // cache them here to retrieve them syncronously in getCellText.
+    this._localShortcutL10nNames = new Map();
+    return document.l10n
+      .formatValues(
+        UrlbarUtils.LOCAL_SEARCH_MODES.map(mode => {
+          let name = UrlbarUtils.getResultSourceName(mode.source);
+          return { id: `urlbar-search-mode-${name}` };
+        })
+      )
+      .then(names => {
+        for (let { source } of UrlbarUtils.LOCAL_SEARCH_MODES) {
+          this._localShortcutL10nNames.set(source, names.shift());
+        }
+        // Invalidate the tree now that we have the names in case getCellText was
+        // called before name retrieval finished.
+        this.invalidate();
+      });
+  },
+
   get lastEngineIndex() {
     return this._engineStore.engines.length - 1;
   },
@@ -895,12 +878,17 @@ EngineView.prototype = {
   },
 
   isEngineSelectedAndRemovable() {
+    let defaultEngine = Services.search.defaultEngine;
+    let defaultPrivateEngine = Services.search.defaultPrivateEngine;
     // We don't allow the last remaining engine to be removed, thus the
     // `this.lastEngineIndex != 0` check.
+    // We don't allow the default engine to be removed.
     return (
       this.selectedIndex != -1 &&
       this.lastEngineIndex != 0 &&
-      !this._getLocalShortcut(this.selectedIndex)
+      !this._getLocalShortcut(this.selectedIndex) &&
+      this.selectedEngine.name != defaultEngine.name &&
+      this.selectedEngine.name != defaultPrivateEngine.name
     );
   },
 
