@@ -37,7 +37,7 @@ add_task(async function test_private_popup_window_opens_private_tabs() {
   // First, open a private browsing window, and load our
   // testing page.
   let privBrowser = privWin.gBrowser.selectedBrowser;
-  await BrowserTestUtils.loadURI(privBrowser, WINDOW_BODY);
+  BrowserTestUtils.loadURIString(privBrowser, WINDOW_BODY);
   await BrowserTestUtils.browserLoaded(privBrowser);
 
   // Next, click on the link in the testing page, and ensure
