@@ -1,12 +1,18 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-// Tests the muxer's result groups composition logic: child buckets,
+// Tests the muxer's result groups composition logic: child groups,
 // `availableSpan`, `maxResultCount`, flex, etc. The purpose of this test is to
 // check the composition logic, not every possible result type or group.
 
 "use strict";
 
+const lazy = {};
+
+ChromeUtils.defineESModuleGetters(lazy, {
+  sinon: "resource://testing-common/Sinon.sys.mjs",
+});
+
 // The possible limit-related properties in result groups.
 const LIMIT_KEYS = ["availableSpan", "maxResultCount"];
 
@@ -19,16 +25,22 @@ const LIMIT_KEYS = ["availableSpan", "maxResultCount"];
 // with the actual limit key. This allows us to run checks against each of the
 // limit keys using essentially the same task.
 
-const RESULT_GROUPS_PREF = "browser.urlbar.resultGroups";
 const MAX_RICH_RESULTS_PREF = "browser.urlbar.maxRichResults";
 
 // For simplicity, most of the flex tests below assume that this is 10, so
 // you'll need to update them if you change this.
 const MAX_RESULTS = 10;
 
+let sandbox;
+
 add_task(async function setup() {
   // Set a specific maxRichResults for sanity's sake.
   Services.prefs.setIntPref(MAX_RICH_RESULTS_PREF, MAX_RESULTS);
+
+  sandbox = lazy.sinon.createSandbox();
+  registerCleanupFunction(() => {
+    sandbox.restore();
+  });
 });
 
 add_resultGroupsLimit_tasks({
@@ -1421,13 +1433,15 @@ add_resultGroups_task({
 /**
  * Adds a single test task.
  *
- * @param {string} testName
+ * @param {object} options
+ *   The options for the test
+ * @param {string} options.testName
  *   This name is logged with `info` as the task starts.
- * @param {object} resultGroups
+ * @param {object} options.resultGroups
  *   browser.urlbar.resultGroups is set to this value as the task starts.
- * @param {array} providerResults
+ * @param {Array} options.providerResults
  *   Array of result objects that the test provider will add.
- * @param {array} expectedResultIndexes
+ * @param {Array} options.expectedResultIndexes
  *   Array of indexes in `providerResults` of the expected final results.
  */
 function add_resultGroups_task({
@@ -1456,10 +1470,16 @@ function add_resultGroups_task({
 /**
  * Adds test tasks for each of the keys in `LIMIT_KEYS`.
  *
- * @param {string} testName
- * @param {object} resultGroups
- * @param {array} providerResults
- * @param {array} expectedResultIndexes
+ * @param {object} options
+ *   The options for the test
+ * @param {string} options.testName
+ *   The name of the test.
+ * @param {object} options.resultGroups
+ *   The resultGroups object to set.
+ * @param {Array} options.providerResults
+ *   The results to return from the test
+ * @param {Array} options.expectedResultIndexes
+ *   Indexes of the expected results within {@link providerResults}
  */
 function add_resultGroupsLimit_tasks({
   testName,
@@ -1549,12 +1569,8 @@ function makeIndexRange(startIndex, count) {
 }
 
 function setResultGroups(resultGroups) {
+  sandbox.restore();
   if (resultGroups) {
-    Services.prefs.setCharPref(
-      RESULT_GROUPS_PREF,
-      JSON.stringify(resultGroups)
-    );
-  } else {
-    Services.prefs.clearUserPref(RESULT_GROUPS_PREF);
+    sandbox.stub(UrlbarPrefs, "resultGroups").get(() => resultGroups);
   }
 }
