@@ -61,9 +61,6 @@ nsLineBox::nsLineBox(nsIFrame* aFrame, int32_t aCount, bool aIsBlock)
     NS_ASSERTION(aIsBlock == f->IsBlockOutside(), "wrong kind of child frame");
   }
 #endif
-  static_assert(static_cast<int>(StyleClear::Max) <= 15,
-                "FlagBits needs more bits to store the full range of "
-                "break type ('clear') values");
   mChildCount = aCount;
   MarkDirty();
   mFlags.mBlock = aIsBlock;
@@ -177,53 +174,31 @@ void nsLineBox::Cleanup() {
 
 #ifdef DEBUG_FRAME_DUMP
 static void ListFloats(FILE* out, const char* aPrefix,
-                       const nsFloatCacheList& aFloats) {
-  nsFloatCache* fc = aFloats.Head();
-  while (fc) {
+                       const nsTArray<nsIFrame*>& aFloats) {
+  for (nsIFrame* f : aFloats) {
     nsCString str(aPrefix);
-    nsIFrame* frame = fc->mFloat;
-    str += nsPrintfCString("floatframe@%p ", static_cast<void*>(frame));
-    if (frame) {
-      nsAutoString frameName;
-      frame->GetFrameName(frameName);
-      str += NS_ConvertUTF16toUTF8(frameName).get();
-    } else {
-      str += "\n###!!! NULL out-of-flow frame";
-    }
+    str += nsPrintfCString("floatframe@%p ", static_cast<void*>(f));
+    nsAutoString frameName;
+    f->GetFrameName(frameName);
+    str += NS_ConvertUTF16toUTF8(frameName).get();
     fprintf_stderr(out, "%s\n", str.get());
-    fc = fc->Next();
   }
 }
 
-/* static */ const char* nsLineBox::BreakTypeToString(StyleClear aBreakType) {
-  switch (aBreakType) {
+/* static */ const char* nsLineBox::StyleClearToString(StyleClear aClearType) {
+  switch (aClearType) {
     case StyleClear::None:
-      return "nobr";
+      return "none";
     case StyleClear::Left:
-      return "leftbr";
+      return "left";
     case StyleClear::Right:
-      return "rightbr";
+      return "right";
     case StyleClear::Both:
-      return "leftbr+rightbr";
-    case StyleClear::Line:
-      return "linebr";
-    case StyleClear::Max:
-      return "leftbr+rightbr+linebr";
+      return "both";
   }
   return "unknown";
 }
 
-char* nsLineBox::StateToString(char* aBuf, int32_t aBufSize) const {
-  snprintf(aBuf, aBufSize, "%s,%s,%s,%s,%s,before:%s,after:%s[0x%x]",
-           IsBlock() ? "block" : "inline", IsDirty() ? "dirty" : "clean",
-           IsPreviousMarginDirty() ? "prevmargindirty" : "prevmarginclean",
-           IsImpactedByFloat() ? "impacted" : "not-impacted",
-           IsLineWrapped() ? "wrapped" : "not-wrapped",
-           BreakTypeToString(GetBreakTypeBefore()),
-           BreakTypeToString(GetBreakTypeAfter()), mAllFlags);
-  return aBuf;
-}
-
 void nsLineBox::List(FILE* out, int32_t aIndent,
                      nsIFrame::ListFlags aFlags) const {
   nsCString str;
@@ -236,10 +211,17 @@ void nsLineBox::List(FILE* out, int32_t aIndent,
 void nsLineBox::List(FILE* out, const char* aPrefix,
                      nsIFrame::ListFlags aFlags) const {
   nsCString str(aPrefix);
-  char cbuf[100];
-  str += nsPrintfCString("line@%p count=%d state=%s ",
-                         static_cast<const void*>(this), GetChildCount(),
-                         StateToString(cbuf, sizeof(cbuf)));
+  str += nsPrintfCString(
+      "line@%p count=%d state=%s,%s,%s,%s,%s,%s,clear-before:%s,clear-after:%s",
+      this, GetChildCount(), IsBlock() ? "block" : "inline",
+      IsDirty() ? "dirty" : "clean",
+      IsPreviousMarginDirty() ? "prevmargindirty" : "prevmarginclean",
+      IsImpactedByFloat() ? "impacted" : "not-impacted",
+      IsLineWrapped() ? "wrapped" : "not-wrapped",
+      HasForcedLineBreak() ? "forced-break" : "no-break",
+      StyleClearToString(FloatClearTypeBefore()),
+      StyleClearToString(FloatClearTypeAfter()));
+
   if (IsBlock() && !GetCarriedOutBEndMargin().IsZero()) {
     const nscoord bm = GetCarriedOutBEndMargin().get();
     str += nsPrintfCString("bm=%s ",
@@ -366,8 +348,7 @@ bool nsLineBox::CachedIsEmpty() {
 }
 
 void nsLineBox::DeleteLineList(nsPresContext* aPresContext, nsLineList& aLines,
-                               nsIFrame* aDestructRoot, nsFrameList* aFrames,
-                               PostDestroyData& aPostDestroyData) {
+                               nsFrameList* aFrames, DestroyContext& aContext) {
   PresShell* presShell = aPresContext->PresShell();
 
   // Keep our line list and frame list up to date as we
@@ -384,7 +365,7 @@ void nsLineBox::DeleteLineList(nsPresContext* aPresContext, nsLineList& aLines,
       MOZ_DIAGNOSTIC_ASSERT(child == line->mFirstChild, "Lines out of sync");
       line->mFirstChild = aFrames->FirstChild();
       line->NoteFrameRemoved(child);
-      child->DestroyFrom(aDestructRoot, aPostDestroyData);
+      child->Destroy(aContext);
     }
     MOZ_DIAGNOSTIC_ASSERT(line == aLines.front(),
                           "destroying child frames messed up our lines!");
@@ -466,44 +447,36 @@ void nsLineBox::MaybeFreeData() {
   }
 }
 
-// XXX get rid of this???
-nsFloatCache* nsLineBox::GetFirstFloat() {
-  MOZ_ASSERT(IsInline(), "block line can't have floats");
-  return mInlineData ? mInlineData->mFloats.Head() : nullptr;
-}
-
-// XXX this might be too eager to free memory
-void nsLineBox::FreeFloats(nsFloatCacheFreeList& aFreeList) {
+void nsLineBox::ClearFloats() {
   MOZ_ASSERT(IsInline(), "block line can't have floats");
   if (IsInline() && mInlineData) {
-    if (mInlineData->mFloats.NotEmpty()) {
-      aFreeList.Append(mInlineData->mFloats);
-    }
+    mInlineData->mFloats.Clear();
     MaybeFreeData();
   }
 }
 
-void nsLineBox::AppendFloats(nsFloatCacheFreeList& aFreeList) {
+void nsLineBox::AppendFloats(nsTArray<nsIFrame*>&& aFloats) {
   MOZ_ASSERT(IsInline(), "block line can't have floats");
-  if (IsInline()) {
-    if (aFreeList.NotEmpty()) {
-      if (!mInlineData) {
-        mInlineData = new ExtraInlineData(GetPhysicalBounds());
-      }
-      mInlineData->mFloats.Append(aFreeList);
+  if (MOZ_UNLIKELY(!IsInline())) {
+    return;
+  }
+  if (!aFloats.IsEmpty()) {
+    if (mInlineData) {
+      mInlineData->mFloats.AppendElements(std::move(aFloats));
+    } else {
+      mInlineData = new ExtraInlineData(GetPhysicalBounds());
+      mInlineData->mFloats = std::move(aFloats);
     }
   }
 }
 
 bool nsLineBox::RemoveFloat(nsIFrame* aFrame) {
   MOZ_ASSERT(IsInline(), "block line can't have floats");
+  MOZ_ASSERT(aFrame);
   if (IsInline() && mInlineData) {
-    nsFloatCache* fc = mInlineData->mFloats.Find(aFrame);
-    if (fc) {
+    if (mInlineData->mFloats.RemoveElement(aFrame)) {
       // Note: the placeholder is part of the line's child list
       // and will be removed later.
-      mInlineData->mFloats.Remove(fc);
-      delete fc;
       MaybeFreeData();
       return true;
     }
@@ -633,221 +606,21 @@ nsLineIterator::FindFrameAt(int32_t aLineNumber, nsPoint aPos,
     return NS_OK;
   }
 
-  if (line->ISize() == 0 && line->BSize() == 0) return NS_ERROR_FAILURE;
-
-  nsIFrame* frame = line->mFirstChild;
-  nsIFrame* closestFromStart = nullptr;
-  nsIFrame* closestFromEnd = nullptr;
-
-  WritingMode wm = line->mWritingMode;
-  nsSize containerSize = line->mContainerSize;
-
-  LogicalPoint pos(wm, aPos, containerSize);
+  if (line->ISize() == 0 && line->BSize() == 0) {
+    return NS_ERROR_FAILURE;
+  }
 
+  LineFrameFinder finder(aPos, line->mContainerSize, line->mWritingMode,
+                         mRightToLeft);
   int32_t n = line->GetChildCount();
+  nsIFrame* frame = line->mFirstChild;
   while (n--) {
-    LogicalRect rect = frame->GetLogicalRect(wm, containerSize);
-    if (rect.ISize(wm) > 0) {
-      // If pos.I() is inside this frame - this is it
-      if (rect.IStart(wm) <= pos.I(wm) && rect.IEnd(wm) > pos.I(wm)) {
-        closestFromStart = closestFromEnd = frame;
-        break;
-      }
-      if (rect.IStart(wm) < pos.I(wm)) {
-        if (!closestFromStart ||
-            rect.IEnd(wm) >
-                closestFromStart->GetLogicalRect(wm, containerSize).IEnd(wm))
-          closestFromStart = frame;
-      } else {
-        if (!closestFromEnd ||
-            rect.IStart(wm) <
-                closestFromEnd->GetLogicalRect(wm, containerSize).IStart(wm))
-          closestFromEnd = frame;
-      }
+    finder.Scan(frame);
+    if (finder.IsDone()) {
+      break;
     }
     frame = frame->GetNextSibling();
   }
-  if (!closestFromStart && !closestFromEnd) {
-    // All frames were zero-width. Just take the first one.
-    closestFromStart = closestFromEnd = line->mFirstChild;
-  }
-  *aPosIsBeforeFirstFrame = mRightToLeft ? !closestFromEnd : !closestFromStart;
-  *aPosIsAfterLastFrame = mRightToLeft ? !closestFromStart : !closestFromEnd;
-  if (closestFromStart == closestFromEnd) {
-    *aFrameFound = closestFromStart;
-  } else if (!closestFromStart) {
-    *aFrameFound = closestFromEnd;
-  } else if (!closestFromEnd) {
-    *aFrameFound = closestFromStart;
-  } else {  // we're between two frames
-    nscoord delta =
-        closestFromEnd->GetLogicalRect(wm, containerSize).IStart(wm) -
-        closestFromStart->GetLogicalRect(wm, containerSize).IEnd(wm);
-    if (pos.I(wm) <
-        closestFromStart->GetLogicalRect(wm, containerSize).IEnd(wm) +
-            delta / 2) {
-      *aFrameFound = closestFromStart;
-    } else {
-      *aFrameFound = closestFromEnd;
-    }
-  }
+  finder.Finish(aFrameFound, aPosIsBeforeFirstFrame, aPosIsAfterLastFrame);
   return NS_OK;
 }
-
-//----------------------------------------------------------------------
-
-#ifdef NS_BUILD_REFCNT_LOGGING
-nsFloatCacheList::nsFloatCacheList() : mHead(nullptr) {
-  MOZ_COUNT_CTOR(nsFloatCacheList);
-}
-#endif
-
-nsFloatCacheList::~nsFloatCacheList() {
-  DeleteAll();
-  MOZ_COUNT_DTOR(nsFloatCacheList);
-}
-
-void nsFloatCacheList::DeleteAll() {
-  nsFloatCache* c = mHead;
-  while (c) {
-    nsFloatCache* next = c->Next();
-    delete c;
-    c = next;
-  }
-  mHead = nullptr;
-}
-
-nsFloatCache* nsFloatCacheList::Tail() const {
-  nsFloatCache* fc = mHead;
-  while (fc) {
-    if (!fc->mNext) {
-      break;
-    }
-    fc = fc->mNext;
-  }
-  return fc;
-}
-
-void nsFloatCacheList::Append(nsFloatCacheFreeList& aList) {
-  MOZ_ASSERT(aList.NotEmpty(), "Appending empty list will fail");
-
-  nsFloatCache* tail = Tail();
-  if (tail) {
-    NS_ASSERTION(!tail->mNext, "Bogus!");
-    tail->mNext = aList.mHead;
-  } else {
-    NS_ASSERTION(!mHead, "Bogus!");
-    mHead = aList.mHead;
-  }
-  aList.mHead = nullptr;
-  aList.mTail = nullptr;
-}
-
-nsFloatCache* nsFloatCacheList::Find(nsIFrame* aOutOfFlowFrame) {
-  nsFloatCache* fc = mHead;
-  while (fc) {
-    if (fc->mFloat == aOutOfFlowFrame) {
-      break;
-    }
-    fc = fc->Next();
-  }
-  return fc;
-}
-
-nsFloatCache* nsFloatCacheList::RemoveAndReturnPrev(nsFloatCache* aElement) {
-  nsFloatCache* fc = mHead;
-  nsFloatCache* prev = nullptr;
-  while (fc) {
-    if (fc == aElement) {
-      if (prev) {
-        prev->mNext = fc->mNext;
-      } else {
-        mHead = fc->mNext;
-      }
-      return prev;
-    }
-    prev = fc;
-    fc = fc->mNext;
-  }
-  return nullptr;
-}
-
-//----------------------------------------------------------------------
-
-#ifdef NS_BUILD_REFCNT_LOGGING
-nsFloatCacheFreeList::nsFloatCacheFreeList() : mTail(nullptr) {
-  MOZ_COUNT_CTOR(nsFloatCacheFreeList);
-}
-
-nsFloatCacheFreeList::~nsFloatCacheFreeList() {
-  MOZ_COUNT_DTOR(nsFloatCacheFreeList);
-}
-#endif
-
-void nsFloatCacheFreeList::Append(nsFloatCacheList& aList) {
-  MOZ_ASSERT(aList.NotEmpty(), "Appending empty list will fail");
-
-  if (mTail) {
-    NS_ASSERTION(!mTail->mNext, "Bogus");
-    mTail->mNext = aList.mHead;
-  } else {
-    NS_ASSERTION(!mHead, "Bogus");
-    mHead = aList.mHead;
-  }
-  mTail = aList.Tail();
-  aList.mHead = nullptr;
-}
-
-void nsFloatCacheFreeList::Remove(nsFloatCache* aElement) {
-  nsFloatCache* prev = nsFloatCacheList::RemoveAndReturnPrev(aElement);
-  if (mTail == aElement) {
-    mTail = prev;
-  }
-}
-
-void nsFloatCacheFreeList::DeleteAll() {
-  nsFloatCacheList::DeleteAll();
-  mTail = nullptr;
-}
-
-nsFloatCache* nsFloatCacheFreeList::Alloc(nsIFrame* aFloat) {
-  MOZ_ASSERT(aFloat->HasAnyStateBits(NS_FRAME_OUT_OF_FLOW),
-             "This is a float cache, why isn't the frame out-of-flow?");
-
-  nsFloatCache* fc = mHead;
-  if (mHead) {
-    if (mHead == mTail) {
-      mHead = mTail = nullptr;
-    } else {
-      mHead = fc->mNext;
-    }
-    fc->mNext = nullptr;
-  } else {
-    fc = new nsFloatCache();
-  }
-  fc->mFloat = aFloat;
-  return fc;
-}
-
-void nsFloatCacheFreeList::Append(nsFloatCache* aFloat) {
-  NS_ASSERTION(!aFloat->mNext, "Bogus!");
-  aFloat->mNext = nullptr;
-  if (mTail) {
-    NS_ASSERTION(!mTail->mNext, "Bogus!");
-    mTail->mNext = aFloat;
-    mTail = aFloat;
-  } else {
-    NS_ASSERTION(!mHead, "Bogus!");
-    mHead = mTail = aFloat;
-  }
-}
-
-//----------------------------------------------------------------------
-
-nsFloatCache::nsFloatCache() : mFloat(nullptr), mNext(nullptr) {
-  MOZ_COUNT_CTOR(nsFloatCache);
-}
-
-#ifdef NS_BUILD_REFCNT_LOGGING
-nsFloatCache::~nsFloatCache() { MOZ_COUNT_DTOR(nsFloatCache); }
-#endif
