@@ -13,11 +13,13 @@
 var helpers = require("../helpers");
 
 module.exports = {
+  // This rule currently has no messages.
+  // eslint-disable-next-line eslint-plugin/prefer-message-ids
   meta: {
     docs: {
-      url:
-        "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/mark-test-function-used.html",
+      url: "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/mark-test-function-used.html",
     },
+    schema: [],
     type: "problem",
   },
 
