@@ -18,11 +18,11 @@ const componentsMap = {
 module.exports = {
   meta: {
     docs: {
-      url:
-        "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/use-cc-etc.html",
+      url: "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/use-cc-etc.html",
     },
-    type: "suggestion",
     fixable: "code",
+    schema: [],
+    type: "suggestion",
   },
 
   create(context) {
