@@ -24,10 +24,10 @@ function isMemberExpression(node, object, member) {
 module.exports = {
   meta: {
     docs: {
-      url:
-        "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/use-chromeutils-import.html",
+      url: "https://firefox-source-docs.mozilla.org/code-quality/lint/linters/eslint-plugin-mozilla/use-chromeutils-import.html",
     },
     fixable: "code",
+    schema: [],
     type: "suggestion",
   },
 
