@@ -10,14 +10,14 @@ They are added to 'try_task_config.json' and processed by the transforms.
 
 import json
 import os
-import six
 import subprocess
 import sys
 from abc import ABCMeta, abstractmethod, abstractproperty
-from argparse import Action, SUPPRESS
+from argparse import SUPPRESS, Action
 from textwrap import dedent
 
 import mozpack.path as mozpath
+import six
 from mozbuild.base import BuildEnvironmentNotFoundException, MozbuildObject
 
 from .tasks import resolve_tests_by_suite
@@ -78,14 +78,14 @@ class Artifact(TryConfig):
 
     def try_config(self, artifact, no_artifact, **kwargs):
         if artifact:
-            return {"use-artifact-builds": True}
+            return {"use-artifact-builds": True, "disable-pgo": True}
 
         if no_artifact:
             return
 
         if self.is_artifact_build():
             print("Artifact builds enabled, pass --no-artifact to disable")
-            return {"use-artifact-builds": True}
+            return {"use-artifact-builds": True, "disable-pgo": True}
 
 
 class Pernosco(TryConfig):
@@ -114,6 +114,7 @@ class Pernosco(TryConfig):
         return super().add_arguments(group)
 
     def try_config(self, pernosco, **kwargs):
+        pernosco = pernosco or os.environ.get("MOZ_USE_PERNOSCO")
         if pernosco is None:
             return
 
@@ -158,9 +159,12 @@ class Pernosco(TryConfig):
                         break
 
         return {
+            "pernosco": True,
+            # TODO Bug 1907076: Remove the env below once Pernosco consumers
+            # are using the `pernosco-v1` task routes.
             "env": {
                 "PERNOSCO": str(int(pernosco)),
-            }
+            },
         }
 
     def validate(self, **kwargs):
@@ -472,8 +476,8 @@ class WorkerOverrides(TryConfig):
     ]
 
     def try_config(self, worker_overrides, worker_suffixes, **kwargs):
+        from gecko_taskgraph.util.workertypes import get_worker_type
         from taskgraph.config import load_graph_config
-        from taskgraph.util.workertypes import get_worker_type
 
         overrides = {}
         if worker_overrides:
@@ -505,10 +509,7 @@ class WorkerOverrides(TryConfig):
                     )
                     sys.exit(1)
                 provisioner, worker_type = get_worker_type(
-                    graph_config,
-                    alias,
-                    level="1",
-                    release_level="staging",
+                    graph_config, worker_type=alias, parameters={"level": "1"}
                 )
                 overrides[alias] = "{provisioner}/{worker_type}{suffix}".format(
                     provisioner=provisioner, worker_type=worker_type, suffix=suffix
