@@ -10,16 +10,15 @@ import sys
 from collections import defaultdict
 
 import mozpack.path as mozpath
+import taskgraph
 from mach.util import get_state_dir
 from mozbuild.base import MozbuildObject
 from mozpack.files import FileFinder
-from moztest.resolve import TestResolver, TestManifestLoader, get_suite_definition
+from moztest.resolve import TestManifestLoader, TestResolver, get_suite_definition
+from taskgraph.generator import TaskGraphGenerator
 from taskgraph.parameters import ParameterMismatch, parameters_loader
 from taskgraph.taskgraph import TaskGraph
 
-import gecko_taskgraph
-from gecko_taskgraph.generator import TaskGraphGenerator
-
 here = os.path.abspath(os.path.dirname(__file__))
 build = MozbuildObject.from_environment(cwd=here)
 
@@ -80,7 +79,7 @@ def generate_tasks(params=None, full=False, disable_target_task_filter=False):
         },
     )
     root = os.path.join(build.topsrcdir, "taskcluster", "ci")
-    gecko_taskgraph.fast = True
+    taskgraph.fast = True
     generator = TaskGraphGenerator(root_dir=root, parameters=params)
 
     cache_dir = os.path.join(
