@@ -14,30 +14,27 @@ QUIET=0
 # By default just assume that these tools exist on our path
 MAR=${MAR:-mar}
 MBSDIFF=${MBSDIFF:-mbsdiff}
-if [[ -z "${MAR_OLD_FORMAT}" ]]; then
-  XZ=${XZ:-xz}
+XZ=${XZ:-xz}
+$XZ --version > /dev/null 2>&1
+if [ $? -ne 0 ]; then
+  # If $XZ is not set and not found on the path then this is probably
+  # running on a windows buildbot. Some of the Windows build systems have
+  # xz.exe in topsrcdir/xz/. Look in the places this would be in both a
+  # mozilla-central and comm-central build.
+  XZ="$(dirname "$(dirname "$(dirname "$0")")")/xz/xz.exe"
   $XZ --version > /dev/null 2>&1
   if [ $? -ne 0 ]; then
-    # If $XZ is not set and not found on the path then this is probably
-    # running on a windows buildbot. Some of the Windows build systems have
-    # xz.exe in topsrcdir/xz/. Look in the places this would be in both a
-    # mozilla-central and comm-central build.
-    XZ="$(dirname "$(dirname "$(dirname "$0")")")/xz/xz.exe"
+    XZ="$(dirname "$(dirname "$(dirname "$(dirname "$0")")")")/xz/xz.exe"
     $XZ --version > /dev/null 2>&1
     if [ $? -ne 0 ]; then
-      XZ="$(dirname "$(dirname "$(dirname "$(dirname "$0")")")")/xz/xz.exe"
-      $XZ --version > /dev/null 2>&1
-      if [ $? -ne 0 ]; then
-        echo "xz was not found on this system!"
-        echo "exiting"
-        exit 1
-      fi
+      echo "xz was not found on this system!"
+      echo "exiting"
+      exit 1
     fi
   fi
-else
-  MAR_OLD_FORMAT=1
-  BZIP2=${BZIP2:-bzip2}
 fi
+# Ensure that we're always using the right compression settings
+export XZ_OPT="-T1 -7e"
 
 # -----------------------------------------------------------------------------
 # Helper routines
@@ -70,13 +67,7 @@ copy_perm() {
 
 make_add_instruction() {
   f="$1"
-  filev2="$2"
-  # The third param will be an empty string when a file add instruction is only
-  # needed in the version 2 manifest. This only happens when the file has an
-  # add-if-not instruction in the version 3 manifest. This is due to the
-  # precomplete file prior to the version 3 manifest having a remove instruction
-  # for this file so the file is removed before applying a complete update.
-  filev3="$3"
+  filev3="$2"
 
   # Used to log to the console
   if [ $4 ]; then
@@ -91,16 +82,10 @@ make_add_instruction() {
     # before performing this add instruction.
     testdir=$(echo "$f" | sed 's/\(.*distribution\/extensions\/[^\/]*\)\/.*/\1/')
     verbose_notice "     add-if \"$testdir\" \"$f\""
-    echo "add-if \"$testdir\" \"$f\"" >> "$filev2"
-    if [ ! $filev3 = "" ]; then
-      echo "add-if \"$testdir\" \"$f\"" >> "$filev3"
-    fi
+    echo "add-if \"$testdir\" \"$f\"" >> "$filev3"
   else
     verbose_notice "        add \"$f\"$forced"
-    echo "add \"$f\"" >> "$filev2"
-    if [ ! "$filev3" = "" ]; then
-      echo "add \"$f\"" >> "$filev3"
-    fi
+    echo "add \"$f\"" >> "$filev3"
   fi
 }
 
@@ -116,17 +101,6 @@ check_for_add_if_not_update() {
   return 1;
 }
 
-check_for_add_to_manifestv2() {
-  add_if_not_file_chk="$1"
-
-  if [ `basename $add_if_not_file_chk` = "update-settings.ini" ]; then
-    ## "true" *giggle*
-    return 0;
-  fi
-  ## 'false'... because this is bash. Oh yay!
-  return 1;
-}
-
 make_add_if_not_instruction() {
   f="$1"
   filev3="$2"
@@ -137,8 +111,7 @@ make_add_if_not_instruction() {
 
 make_patch_instruction() {
   f="$1"
-  filev2="$2"
-  filev3="$3"
+  filev3="$2"
 
   is_extension=$(echo "$f" | grep -c 'distribution/extensions/.*/')
   if [ $is_extension = "1" ]; then
@@ -146,19 +119,16 @@ make_patch_instruction() {
     # before performing this add instruction.
     testdir=$(echo "$f" | sed 's/\(.*distribution\/extensions\/[^\/]*\)\/.*/\1/')
     verbose_notice "   patch-if \"$testdir\" \"$f.patch\" \"$f\""
-    echo "patch-if \"$testdir\" \"$f.patch\" \"$f\"" >> "$filev2"
     echo "patch-if \"$testdir\" \"$f.patch\" \"$f\"" >> "$filev3"
   else
     verbose_notice "      patch \"$f.patch\" \"$f\""
-    echo "patch \"$f.patch\" \"$f\"" >> "$filev2"
     echo "patch \"$f.patch\" \"$f\"" >> "$filev3"
   fi
 }
 
 append_remove_instructions() {
   dir="$1"
-  filev2="$2"
-  filev3="$3"
+  filev3="$2"
 
   if [ -f "$dir/removed-files" ]; then
     listfile="$dir/removed-files"
@@ -180,17 +150,14 @@ append_remove_instructions() {
         if [ ! $(echo "$f" | grep -c '^#') = 1 ]; then
           if [ $(echo "$f" | grep -c '\/$') = 1 ]; then
             verbose_notice "      rmdir \"$f\""
-            echo "rmdir \"$f\"" >> "$filev2"
             echo "rmdir \"$f\"" >> "$filev3"
           elif [ $(echo "$f" | grep -c '\/\*$') = 1 ]; then
             # Remove the *
             f=$(echo "$f" | sed -e 's:\*$::')
             verbose_notice "    rmrfdir \"$f\""
-            echo "rmrfdir \"$f\"" >> "$filev2"
             echo "rmrfdir \"$f\"" >> "$filev3"
           else
             verbose_notice "     remove \"$f\""
-            echo "remove \"$f\"" >> "$filev2"
             echo "remove \"$f\"" >> "$filev3"
           fi
         fi
